import { z } from 'zod';
import { emailParserAnyCase } from '../../../common/parsers/email';
import { entropy } from '../../../utils/string/entropy';

const ENTROPY_THRESHOLD = 50; // pretty weak
export const passwordEntropyParser = z.string().superRefine((s, ctx) => {
  const en = entropy(s);
  if (en >= ENTROPY_THRESHOLD) return;
  ctx.addIssue({
    code: z.ZodIssueCode.custom,
    params: {
      entropy: en,
      required: ENTROPY_THRESHOLD,
      code: 'entropy',
    },
    message: `Password entropy is too low (${en}). Minimum entropy is ${ENTROPY_THRESHOLD}`,
  });
});
const passwordParser = z.string().min(8).and(passwordEntropyParser);
export const signupRequestParser = z.object({
  email: emailParserAnyCase,
  password: passwordParser,
});
