"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.authUsersListResultParser = exports.authUserParser = exports.authUsersGetOneResultParser = exports.authUsersGetOneParamsParser = exports.userIdParser = exports.authUsersListQueryParser = exports.authUsersListPageTokenParamsParser = exports.authUsersListLimitParamsParser = exports.queryPageTokenLimitParser = exports.makePageTokenLimitParser = exports.makePageTokenParser = exports.authUsersQueryPageTokenParser = exports.authUsersQueryLimitParser = void 0;
const zod_1 = require("zod");
const constants_1 = require("./constants");
const id_utils_1 = require("./id.utils");
const queryParams_1 = require("../../common/parsers/queryParams");
const types_1 = require("../../utils/types");
// for 'get' query params, .or shenanigans
exports.authUsersQueryLimitParser = queryParams_1.queryLimitParser.refine((n) => n <= constants_1.AUTH_USERS_API_GET_LIMIT, {
    message: `limit must be less than ${constants_1.AUTH_USERS_API_GET_LIMIT}`,
});
exports.authUsersQueryPageTokenParser = zod_1.z.string().refine((n) => n, {
    message: 'page token must be a string',
});
const makePageTokenParser = (parser) => zod_1.z.object({
    pageToken: parser,
});
exports.makePageTokenParser = makePageTokenParser;
const makePageTokenLimitParser = (limitParser, pageTokenParser) => (0, queryParams_1.makeLimitParser)(limitParser).merge((0, exports.makePageTokenParser)(pageTokenParser));
exports.makePageTokenLimitParser = makePageTokenLimitParser;
exports.queryPageTokenLimitParser = (0, exports.makePageTokenLimitParser)(exports.authUsersQueryPageTokenParser, queryParams_1.queryLimitParser);
const authUsersListLimitParser = exports.authUsersQueryLimitParser.optional().default(constants_1.AUTH_USERS_API_GET_LIMIT);
const authUsersListPageTokenParser = exports.authUsersQueryPageTokenParser.optional();
exports.authUsersListLimitParamsParser = (0, queryParams_1.makeLimitParser)(authUsersListLimitParser);
exports.authUsersListPageTokenParamsParser = (0, exports.makePageTokenParser)(authUsersListPageTokenParser);
// TODO test that limit can't leak over 1000 or to -1
exports.authUsersListQueryParser = (0, exports.makePageTokenLimitParser)(authUsersListLimitParser, authUsersListPageTokenParser);
exports.userIdParser = zod_1.z.string().refine(id_utils_1.isUserId);
exports.authUsersGetOneParamsParser = zod_1.z.object({
    id: exports.userIdParser,
});
exports.authUsersGetOneResultParser = zod_1.z.any( /*stub, not schemable*/).transform((0, types_1.cast)());
exports.authUserParser = zod_1.z.object({
    uid: exports.userIdParser,
    email: zod_1.z.string().email().optional(),
});
exports.authUsersListResultParser = zod_1.z.any().transform((0, types_1.cast)());
