"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.payinApiDstCompositeParser = exports.payinApiDstSerializedParser = exports.payinApiSrcCompositeParser = exports.payinApiSrcSerializedParser = exports.SEPARATOR = exports.payinCompositeIdParser = exports.payinOriginIdParser = exports.payinOriginTypeParser = void 0;
const zod_1 = __importDefault(require("zod"));
const constants_1 = require("../origin/constants");
const parsers_1 = require("../stripe/parsers");
const constants_2 = require("./constants");
const types_1 = require("../utils/types");
exports.payinOriginTypeParser = zod_1.default.enum(constants_2.PAYIN_ORIGIN_TYPES);
exports.payinOriginIdParser = parsers_1.stripePaymentIntentIdParser;
exports.payinCompositeIdParser = zod_1.default
    .object({
    provider: exports.payinOriginTypeParser,
    originId: exports.payinOriginIdParser,
})
    .and(zod_1.default.discriminatedUnion('provider', [
    zod_1.default.object({
        provider: zod_1.default.literal(constants_1.ORIGIN_TYPE_STRIPE),
        originId: parsers_1.stripePaymentIntentIdParser,
    }),
    zod_1.default.object({
        provider: zod_1.default.literal(constants_1.ORIGIN_TYPE_PAYPAL),
        originId: zod_1.default.literal('paypal'), // TODO
    }),
]));
exports.SEPARATOR = '::'; // just coincides with UUID_SEPARATOR
// "api src" - looks just like a normal graph src but is not graph src; abstractions are just similar
// (in truth, its because the older api already works like that but we'd like to both move from the graph terminology and keep compatibility)
// [8c2eab11-1e88-41dc-a520-ed22af399f96]
const makeApiIdParser = (literals) => {
    const makeType = (literal) => zod_1.default.object({
        type: zod_1.default.literal(literal),
        id: zod_1.default.string().transform((0, types_1.cast)()),
    });
    // to procure type
    const first = makeType(literals[0]);
    return zod_1.default
        .string()
        .min(exports.SEPARATOR.length + 2 /*id and type*/)
        .transform((uuid, ctx) => {
        const [type, id] = uuid.split(exports.SEPARATOR);
        const next = zod_1.default.discriminatedUnion('type', [
            first,
            ...literals.slice(1).map(makeType),
        ]);
        const nextR = next.safeParse({ type, id: id });
        if (!nextR.success) {
            nextR.error.issues.forEach(ctx.addIssue.bind(ctx));
            return zod_1.default.NEVER;
        }
        else {
            return nextR.data;
        }
    });
};
const makeApiIdSerializedParser = (literals) => makeApiIdParser(literals).transform((cid) => `${cid.type}${exports.SEPARATOR}${cid.id}`);
const srcLiterals = [constants_2.PAYIN_USER_API_LITERAL_TYPE];
exports.payinApiSrcSerializedParser = makeApiIdSerializedParser(srcLiterals);
exports.payinApiSrcCompositeParser = makeApiIdParser(srcLiterals);
const dstLiterals = [constants_2.PAYIN_CHANNEL_API_LITERAL_TYPE];
exports.payinApiDstSerializedParser = makeApiIdSerializedParser(dstLiterals);
exports.payinApiDstCompositeParser = makeApiIdParser(dstLiterals);
