"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.routeContextBindBodyParser = exports.chargeRoutingResultIdParser = exports.routeContextInitResultParser = exports.routeContextBodyParser = exports.routeContextDetailsResultParser = exports.routeContextQueryParser = exports.routeContextParamsParser = exports.chargeRoutingContextContentIdParser = exports.chargeRoutingContextIdParser = exports.routePacketResultParser = exports.routePacketQueryParser = void 0;
const zod_1 = require("zod");
const constants_1 = require("../../constants");
const parsers_1 = require("../../currencies/parsers");
const parsers_2 = require("../../channel/parsers");
const parsers_3 = require("../../user/parsers");
const queryParams_1 = require("../../common/parsers/queryParams");
const parsers_4 = require("../../graph/strategy/parsers");
const constants_2 = require("./constants");
const parsers_5 = require("../../parsers");
const parsers_6 = require("../../firebase/id/parsers");
const parsers_7 = require("../../stripe/parsers");
exports.routePacketQueryParser = zod_1.z
    .object({
    userId: parsers_3.userIdParser,
    payeeId: parsers_2.channelIdParser,
    amt: parsers_1.currencyAmountParser,
    unit: parsers_1.currencyParser,
    when: (0, queryParams_1.makePositiveIntegerQueryParser)('When')
        .refine((n) => n <= constants_1.MAX_HISTORICAL_DATE_MS, {
        message: `"When" must be less than ${constants_1.MAX_HISTORICAL_DATE_MS}`,
    })
        .optional(),
})
    .merge(parsers_4.routingStrategyQueryParser);
exports.routePacketResultParser = zod_1.z.any( /* stub */).transform((v) => v);
exports.chargeRoutingContextIdParser = zod_1.z
    .string()
    .regex(parsers_6.FBIDHarderRegex)
    .transform((s) => s);
exports.chargeRoutingContextContentIdParser = parsers_2.channelIdParser.transform((s) => s); // TODO more
exports.routeContextParamsParser = zod_1.z.object({
    [constants_2.CHARGE_ROUTING_CONTEXT_ID_ROUTE_PARAM_NAME]: exports.chargeRoutingContextIdParser,
});
exports.routeContextQueryParser = zod_1.z.object({
    from: parsers_5.timestampQueryParser.optional(),
});
exports.routeContextDetailsResultParser = exports.routePacketResultParser.and(zod_1.z.object({
    contentId: exports.chargeRoutingContextContentIdParser.nullable(),
    activatedAt: parsers_5.timestampParser.nullable(),
    from: parsers_5.timestampParser,
    to: parsers_5.timestampParser,
}));
exports.routeContextBodyParser = zod_1.z.object({
    contentId: exports.chargeRoutingContextContentIdParser,
    to: parsers_5.timestampQueryParser.optional(),
});
exports.routeContextInitResultParser = zod_1.z.object({
    id: exports.chargeRoutingContextIdParser,
});
exports.chargeRoutingResultIdParser = parsers_7.stripePaymentIntentIdParser; // TODO more
exports.routeContextBindBodyParser = zod_1.z.object({
    graphId: exports.chargeRoutingResultIdParser,
});
