"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.partition = exports.getOnlyOneC = exports.getOnlyOne = exports.assertExactLength = void 0;
const assertExactLength = (length, e0) => {
    if (length < 0)
        throw new Error('length must be positive');
    return (a, e) => {
        if (a.length !== length)
            throw new Error(e0 || e || `length must be ${length}`);
        return a;
    };
};
exports.assertExactLength = assertExactLength;
const getOnlyOne = (a, e) => {
    if (a.length !== 1)
        throw new Error(e || `getOnlyOne expects 1 element, got ${a.length}`);
    return a[0];
};
exports.getOnlyOne = getOnlyOne;
const getOnlyOneC = (e) => (a) => (0, exports.getOnlyOne)(a, e);
exports.getOnlyOneC = getOnlyOneC;
// from "just", MIT
/*
    partition([1, 5, 2, 4, 3], n => n > 3); // [[5, 4],[1, 2, 3]]
    partition(['a', 1, 2, 'b'], x => typeof x == 'string'); // [['a', '3'],[2, 3]]
    partition([1, 2, 3, 4], x => typeof x == 'number'); // [[1, 2, 3, 4],[]]
    partition([1, 2, 3, 4], x => typeof x == 'string'); // [[], [1, 2, 3, 4]]
    partition([], n => n > 3); // [[], []]
    partition({a: 1, b: 2}, n => n > 1); // throws
    partition(null, n => n > 1); // throws
    partition(undefined, n => n > 1); // throws
*/
const partition = (arr, fn) => {
    if (!Array.isArray(arr)) {
        throw new Error('expected first argument to be an array');
    }
    if (typeof fn != 'function') {
        throw new Error('expected second argument to be a function');
    }
    const first = [];
    const second = [];
    const length = arr.length;
    for (let i = 0; i < length; i++) {
        const nextValue = arr[i];
        if (fn(nextValue)) {
            first.push(nextValue);
        }
        else {
            second.push(nextValue);
        }
    }
    return [first, second];
};
exports.partition = partition;
