import { StripePaymentIntentId } from '../stripe/types';
import { ORIGIN_TYPE_PAYPAL, ORIGIN_TYPE_STRIPE, ORIGIN_TYPES_EXTERNAL } from '../origin/constants';
import { Subset } from '../utils/types';
import { PAYIN_CHANNEL_API_LITERAL_TYPE, PAYIN_USER_API_LITERAL_TYPE } from './constants';
import { payinApiSrcDstCompositeParserOptional } from './parsers';
import { UserId } from '../user/types';
import { ChannelIdOfType, ChannelType } from '../channel/types';
import z from 'zod';
import { TipStatus } from '../tip/types';
import { SubscriptionStatus } from '../subscription/types';
import { rerunImpossiblePaymentIntentParamsParser } from './stripe/impossiblePaymentIntent/parsers';
import { PayinProviderOriginType } from '../payinProvider/types';
import {
  payinApiDstCompositeParser,
  payinApiSrcCompositeParser,
  payinCompositeIdParser,
  SEPARATOR,
} from './parsers.common';

export type PayinOriginType = (typeof ORIGIN_TYPES_EXTERNAL)[number];
export type TipOriginId = StripePaymentIntentId;
export type SubscriptionPaymentOriginId = StripePaymentIntentId;
export type PayinOriginIdMapping = {
  [ORIGIN_TYPE_STRIPE]: TipOriginId | SubscriptionPaymentOriginId;
  [ORIGIN_TYPE_PAYPAL]: 'paypal'; // TODO transaction id? no paypal payins at the moment really
};
// eslint-disable-next-line
type AssertPayinOriginIdMapping = Subset<
  PayinOriginIdMapping,
  {
    [k in PayinOriginType]: unknown;
  }
>;
export type PayinOriginId<T extends PayinOriginType = PayinOriginType> = PayinOriginIdMapping[T];
export enum PayinSrcBrand {
  _ = '',
}
export type PayinSrc = `${typeof PAYIN_USER_API_LITERAL_TYPE}${typeof SEPARATOR}${UserId}`; // & PayinSrcBrand makes the type never??
export enum PayinDstBrand {
  _ = '',
}
export type PayinDst<T extends ChannelType = ChannelType> =
  `${typeof PAYIN_CHANNEL_API_LITERAL_TYPE}${typeof SEPARATOR}${ChannelIdOfType<T>}`; // & PayinDstBrand makes the type never??;
export type ApiPayinSrcComposite = z.infer<typeof payinApiSrcCompositeParser>;
export type ApiPayinDstComposite = z.infer<typeof payinApiDstCompositeParser>;

export interface ClientSecretMixin {
  // a secret to follow thru with subscription payment process; user in Stripe;
  client_secret?: string;
  // updated field
  secret?: string;
}

// frontend currently handles both tip and subscription creations the same way, expecting the same shape
// these are the fields that are common to both and used by the frontend
export type ApiPayinResultMixin<
  T extends 'tip' | 'subscription',
  P extends PayinProviderOriginType = PayinProviderOriginType
> = {
  status: T extends 'tip' ? TipStatus : T extends 'subscription' ? SubscriptionStatus : never;
} & ClientSecretMixin &
  PayinCompositeId<P>;

export type PayinSrcDstApiQueryMixin = z.infer<typeof payinApiSrcDstCompositeParserOptional>;

export type RerunImpossiblePaymentIntentParams = z.input<typeof rerunImpossiblePaymentIntentParamsParser>;
export type RerunImpossiblePaymentIntentParamsTransformed = z.output<typeof rerunImpossiblePaymentIntentParamsParser>;

export type PayinCompositeId<P extends PayinProviderOriginType = PayinProviderOriginType> = {
  provider: P;
  originId: PayinOriginId<P>;
};

export type PayinCompositeIdSerialized<P extends PayinProviderOriginType = PayinProviderOriginType> =
  `${PayinCompositeId<P>['provider']}${typeof SEPARATOR}${PayinCompositeId<P>['originId']}`;

type PayinCompositeIdParsed = z.infer<typeof payinCompositeIdParser>;

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertPayinCompositeIdParsed1 = Subset<PayinCompositeIdParsed, PayinCompositeId>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
// type AssertPayinCompositeIdParsed2 = Subset<PayinCompositeId, PayinCompositeIdParsed>;
