import z from 'zod';
import { UserId } from '../user/types';
import { PayoutMethodOriginType } from '../payoutMethod/types';
import { Currency } from '../currencies/types';
import { payoutBodyParser, payoutParamsParser, payoutResultParser, payoutRetryBodyParser } from './parsers';
import { Subset } from '../utils/types';
import {
  PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES,
  PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES,
  PAYOUT_QUEUE_STATUSES,
} from './constants';

export enum PayoutBatchKeyBrand {
  _ = '',
}

// an uniq key per payout batch operation; serves as an idempotency key for a payout
export type PayoutBatchKey = string & PayoutBatchKeyBrand;

export type PayoutBatchItemCompositeKeySerialized<T extends PayoutMethodOriginType = PayoutMethodOriginType> =
  `${UserId}:${T}:${Currency}`;
export type PayoutMethodOriginTypeLineGranular = PayoutMethodOriginType &
  (typeof PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES)[number];
export type PayoutMethodOriginTypeNonLineGranular = PayoutMethodOriginType &
  (typeof PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES)[number];
export type PayoutMethodOriginTypeLineNonGranular = Exclude<PayoutMethodOriginType, PayoutMethodOriginTypeLineGranular>;
export type PayoutBatchItemLineableCompositeKeySerialized =
  PayoutBatchItemCompositeKeySerialized<PayoutMethodOriginTypeLineGranular>;
export type PayoutBatchItemNonLineableCompositeKeySerialized =
  PayoutBatchItemCompositeKeySerialized<PayoutMethodOriginTypeLineNonGranular>;

export interface PayoutBatchLineItemCompositeKey<T extends PayoutMethodOriginType = PayoutMethodOriginType> {
  userId: UserId;
  originType: T;
  currency: Currency;
}

export type PayoutBody = z.input<typeof payoutBodyParser>;
export type PayoutBodyTransformed = z.output<typeof payoutBodyParser>;
export type PayoutRetryBody = z.input<typeof payoutRetryBodyParser>;
export type PayoutRetryBodyTransformed = z.output<typeof payoutRetryBodyParser>;
export type PayoutParams = z.input<typeof payoutParamsParser>;
export type PayoutParamsTransformed = z.output<typeof payoutParamsParser>;
export type PayoutResult = z.infer<typeof payoutResultParser>;

// not the "whole args", doesnt include the batch key which isn't the part of the "args" language for idempotency
export type AuthorPayoutArgs<OriginType extends PayoutMethodOriginType = PayoutMethodOriginType> = Partial<
  Record<PayoutBatchItemCompositeKeySerialized<OriginType>, number>
>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertAuthorPayoutArgs1 = Subset<PayoutBody['items'], AuthorPayoutArgs>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertAuthorPayoutArgs2 = Subset<AuthorPayoutArgs, PayoutBody['items']>;

// pseudo status for the queue item; signifies "all done"/"any done"/"some done"
export type PayoutQueueItemStatus = (typeof PAYOUT_QUEUE_STATUSES)[number];
