import { z } from 'zod';
import type { SessionStateString } from '../csrf/types';
import { paymentMethodCompositeKeyParamsParser, paymentMethodListLimitParser } from '../paymentMethod/parsers';
import { stripeConnectAccountIdParser } from '../stripe/parsers';
import { Subset } from '../utils/types';
import { PayoutMethodOriginType } from './types';
import { PAYOUT_METHOD_ORIGIN_TYPES } from './constants';
import { ORIGIN_TYPE_PAYPAL, ORIGIN_TYPE_STRIPE } from '../origin/constants';

export const payoutMethodStripeAuthorizeResultParser = z.object({
  stripeConnectUrl: z.string().url(),
});
export const payoutMethodStripeTokenBodyParser = z.object({
  state: z.string().transform((s) => s as SessionStateString),
  code: z.string(), // stripe code
});

export const payoutMethodUpsertParamsParser = paymentMethodCompositeKeyParamsParser;
export const payoutMethodDeleteParamsParser = paymentMethodCompositeKeyParamsParser; // no offset supported (stripe works on cursors also)
export const payoutMethodListQueryParser = paymentMethodListLimitParser;

export const payoutMethodOriginIdParser = stripeConnectAccountIdParser; // TODO .or paypal

export const payoutMethodOriginTypeParser = z.enum(PAYOUT_METHOD_ORIGIN_TYPES);

export const payoutMethodListResultParser = z.array(
  z
    .discriminatedUnion('originType', [
      z.object({
        originType: z.literal(ORIGIN_TYPE_STRIPE),
        originId: payoutMethodOriginIdParser,
      }),
      z.object({
        originType: z.literal(ORIGIN_TYPE_PAYPAL),
        originId: z.literal(ORIGIN_TYPE_PAYPAL), // TODO
      }),
    ])
    .and(
      z.object({
        originType: payoutMethodOriginTypeParser,
      })
    )
);

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertPayoutMethodListResult1 = Subset<
  z.infer<typeof payoutMethodListResultParser>[number]['originType'],
  PayoutMethodOriginType
>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertPayoutMethodListResult2 = Subset<
  PayoutMethodOriginType,
  z.infer<typeof payoutMethodListResultParser>[number]['originType']
>;

export const payoutMethodUpsertResultParser = z.any(); // TODO
