import type { ZodEffects, ZodType } from 'zod';
import z from 'zod';

// generates is(o)-like method from a parser
// const isStripeCustomerId = isFromParser(stripeCustomerIdParser)
// on practice though, TS doesn't seem to recognize "nested" `is ` statements, therefore usage like
// (id: string): id is StripeCustomerId => isFromParser(stripeCustomerIdParser)(id); is needed
export const isFromParser =
  <T extends ZodType, Input, Output extends Input, P extends ZodEffects<T, Output, Input>>(parser: P) =>
  (o: z.input<P>): o is Output =>
    parser.safeParse(o).success;

export const nonEmptyStringParser = z.string().min(1);
